package cz.muni.pa165.authentication.rest;

import cz.muni.pa165.authentication.data.model.ApiKey;
import cz.muni.pa165.authentication.data.model.Role;
import cz.muni.pa165.authentication.service.ApiKeyService;
import io.swagger.v3.oas.annotations.media.Schema;
import java.util.Date;
import java.util.List;
import java.util.Set;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClient;
import org.springframework.security.oauth2.client.annotation.RegisteredOAuth2AuthorizedClient;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "/authenticate")
public class AuthenticationController {

  @Autowired
  ApiKeyService apiKeyService;

  @PostMapping(path = "/create/apikey")
  public ResponseEntity<ApiKey> createApiKey(
      @RegisteredOAuth2AuthorizedClient @Schema(hidden = true)
          OAuth2AuthorizedClient oauth2Client,
      @AuthenticationPrincipal @Schema(hidden = true) OidcUser user) throws IllegalAccessException {

    return ResponseEntity.ok(apiKeyService.createApiKey(oauth2Client, user));
  }

  @GetMapping(path = "/apikey")
  public ResponseEntity<Set<Role>> authenticate(@RequestParam String apiKey) {
    return ResponseEntity.ok(apiKeyService.isAuthenticated(apiKey));
  }

  @DeleteMapping(path = "/apikey/delete")
  public ResponseEntity<String> deleteApiKeys(@RegisteredOAuth2AuthorizedClient @Schema(hidden = true)
                                                    OAuth2AuthorizedClient oauth2Client,
                                              @RequestParam String apiKey) {
    return ResponseEntity.ok(apiKeyService.delete(apiKey, oauth2Client));
  }

  @DeleteMapping(path = "/apikey/deleteAll")
  public ResponseEntity<String> deleteAllApiKeys(@RegisteredOAuth2AuthorizedClient @Schema(hidden = true)
                                                       OAuth2AuthorizedClient oauth2Client) {
    return ResponseEntity.ok(apiKeyService.deleteAll(oauth2Client));
  }

  @PatchMapping(path = "/apikey/grantRole")
  public ResponseEntity<ApiKey> grantRoles(@RegisteredOAuth2AuthorizedClient @Schema(hidden = true)
                                                 OAuth2AuthorizedClient oauth2Client,
                                           @RequestParam String apiKey,
                                           @RequestParam Set<Role> roles) {
    return ResponseEntity.ok(apiKeyService.setRoles(apiKey, roles, oauth2Client));
  }

  @GetMapping(path = "/apikey/getAll")
  public ResponseEntity<List<ApiKey>> getAll(@RegisteredOAuth2AuthorizedClient
                                               @Schema(hidden = true)
                                                   OAuth2AuthorizedClient oauth2Client) {
    return ResponseEntity.ok(apiKeyService.getAll(oauth2Client));
  }
}
